#ifndef _SALLDAES_H
#define _SALLDAES_H
/******************************************************************************
 * FILE PURPOSE:  Macros and definitions private to AES encryption/Decryption
 ******************************************************************************
 * FILE NAME:   salldaes.h  
 *
 * DESCRIPTION: AES Encryption/Decryption related defines
 *
 *
 * (C) Copyright 2009, Texas Instruments Inc.
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/
#include <ti/mas/types/types.h>

#define SALLD_NO_KEY_ERROR     0
#define SALLD_BAD_KEY_MAT      1
#define SALLD_BAD_CIPHER_STATE 2

#define SALLD_ENCRYPT_BLOCK_SIZE_INBYTES  16
#define SALLD_DECRYPT_BLOCK_SIZE_INBYTES  16
#define SALLD_ENCRYPT_BLOCK_SIZE_INWORDS  8
#define SALLD_DECRYPT_BLOCK_SIZE_INWORDS  8

#define SALLD_MAX_AES_KEY_SIZE            60

/******************************************************************************
 * FUNCTION PURPOSE: Derive NR from the key size
 ******************************************************************************
 * DESCRIPTION: Derive AES NR from the given key size in bits
 *
 *    int16_t salld_aes_get_nr(
 *              int16_t   keylen),   Key size in bits
 ******************************************************************************/
int16_t salld_aes_get_nr(int16_t keyLen);


/******************************************************************************
 * FUNCTION PURPOSE: AES Counter Mode operation
 ******************************************************************************
 * DESCRIPTION: Does in place Encryption/Decryption of the Packets
 *
 *    int16_t salld_aes_ctr(
 *              uint32_t *key,     Expanded Round Key Array
 *              int16_t   keylen,   Key size in bits
 *              int16_t   nr,       NR: number of rounds
 *              tword   *iv,      Initialization Vector
 *              tword   *inpkt,   Input packet to do encryption/decryption
 *              int16_t   inputLen, Payload length in bytes
 *              tword   *outpkt)  Output encrypted/decrypted packet
 ******************************************************************************
 *    Algorithm: S(j) = E(key, (IV+j) mod 2^128);
 *               OutPkt(j) = S(j) XOR InPkt(j); for encryption/decryption
 *               
 *
 *                  IV          IV+1          IV+2              IV+L-1
 *                   |           |             |                  |
 *                   |           |             |                  |
 *                   v           v             v                  v
 *               +------+     +------+      +------+           +------+
 *               |      |     |      |      |      |           |      |
 *    key ------>|  AES |     |  AES |      |  AES |  . . .    |  AES |
 *               |      |     |      |      |      |           |      |
 *               +------+     +------+      +------+           +------+
 *                   |           |             |                  |
 *                   |           |             |                  |
 *                   v           v             v                  v
 *                  S(0)        S(1)          S(2)  . . .       S(L-1)
 *
 ******************************************************************************/
/*int16_t salld_aes_ctr(uint32_t *key, int16_t keylen, int16_t nr, tword *iv, tword *inpkt,
                   int16_t inputLen, tword *outpkt);*/
				   
int16_t salld_aes_ctr(uint32_t *key, int16_t keylen, int16_t nr, tword *iv,
					salldAesDesc_t *desc);
					
int16_t salld_aes_ctr_update(salldAesInst_t *inst, salldAesDesc_t *desc);

int16_t salld_aes_ctr_single_segment(uint32_t *key, int16_t keylen, int16_t nr, tword *iv, tword *inpkt,
        int16_t inputLen, tword *outpkt);

int16_t salld_aes_ctr_multi_segment(uint32_t *key, int16_t keylen, int16_t nr, tword *iv, salldAesInst_t * inst, salldAesDesc_t *desc);
/******************************************************************************
 * FUNCTION PURPOSE: AES f8 Mode operation
 ******************************************************************************
 * DESCRIPTION: Does inplace encryption/decryption in f8 mode
 *
 *    int8_t salld_aes_f8(
 *              uint32_t *key,     Expanded Round Key Array
 *              int16_t   keylen,   Key size in bits
 *              int16_t   nr,       NR: number of rounds
 *              tword   *iv,      Initialization Vector
 *              tword   *inpkt,   Input packet to do encryption/decryption
 *              int16_t   inputLen, Payload length in bytes
 *              tword   *outpkt)  Output encrypted/decrypted packet
 ******************************************************************************
 *    Algorithm: S(j) = E(key, IV XOR j XOR S(j-1));
 *               OutPkt(j) = S(j) XOR InPkt(j); for encryption/decryption
 *
 *                  IV
 *                   |
 *                   +-----------+-------------+--  ...     ------+
 *                   |           |             |                  |
 *                   |   j=1 -> (*)    j=2 -> (*)   ...  j=L-1 ->(*)
 *                   |           |             |                  |
 *                   |      +-> (*)       +-> (*)   ...      +-> (*)
 *                   |      |    |        |    |             |    |
 *                   v      |    v        |    v             |    v
 *               +------+   | +------+    | +------+         | +------+
 *               |      |   | |      |    | |      |         | |      |
 *    key ------>|  AES |   | |  AES |    | |  AES |         | |  AES |
 *               |      |   | |      |    | |      |         | |      |
 *               +------+   | +------+    | +------+         | +------+
 *                   |      |    |        |    |             |    |
 *                   +------+    +--------+    +--  ...  ----+    |
 *                   |           |             |                  |
 *                   v           v             v                  v
 *                  S(0)        S(1)          S(2)  . . .       S(L-1)
 *
 ******************************************************************************/
int16_t salld_aes_f8(uint32_t *key, int16_t keylen, int16_t nr, tword *iv, 
						salldAesDesc_t *desc);


#endif
/* nothing past this point */ 
