/******************************************************************************
 * FILE PURPOSE:  AES Decoder Implementation for C64+ Little Endian
 ******************************************************************************
 * FILE NAME:   aesDec64P.c  
 *
 * DESCRIPTION: Core AES Decryption Algorithm Implementation 
 *
 * EXTERNAL COMPILE TIME FLAGS:
 *
 *              aes_ALL_KEY_SIZES:
 *                           If we want to support 192 and 256 bit keys
 *                           along with 128 bit key. (It will cost extra cycles!)
 *
 * The following source was taken from cryptographic software written by the below authors
 * and has been modified and adapted by TI
 *****************************************************************************
 * rijndael-alg-fst.c
 *
 * @version 3.0 (December 2000)
 *
 * Optimised ANSI C code for the Rijndael cipher (now AES)
 *
 * @author Vincent Rijmen <vincent.rijmen@esat.kuleuven.ac.be>
 * @author Antoon Bosselaers <antoon.bosselaers@esat.kuleuven.ac.be>
 * @author Paulo Barreto <paulo.barreto@terra.com.br>
 *
 * This code is hereby placed in the public domain.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHORS ''AS IS'' AND ANY EXPRESS
 * OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHORS OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
 * BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
 * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *****************************************************************************/
#include <stdlib.h>
#include <stdio.h>

/* AES header files */
#include "aes.h"
#include "aes64P.h"

/* Little endian tables */
const tulong aesTd0[] = {
    0x50a7f451U, 0x5365417eU, 0xc3a4171aU, 0x965e273aU, 
    0xcb6bab3bU, 0xf1459d1fU, 0xab58faacU, 0x9303e34bU, 
    0x55fa3020U, 0xf66d76adU, 0x9176cc88U, 0x254c02f5U, 
    0xfcd7e54fU, 0xd7cb2ac5U, 0x80443526U, 0x8fa362b5U, 
    0x495ab1deU, 0x671bba25U, 0x980eea45U, 0xe1c0fe5dU, 
    0x02752fc3U, 0x12f04c81U, 0xa397468dU, 0xc6f9d36bU, 
    0xe75f8f03U, 0x959c9215U, 0xeb7a6dbfU, 0xda595295U, 
    0x2d83bed4U, 0xd3217458U, 0x2969e049U, 0x44c8c98eU, 
    0x6a89c275U, 0x78798ef4U, 0x6b3e5899U, 0xdd71b927U, 
    0xb64fe1beU, 0x17ad88f0U, 0x66ac20c9U, 0xb43ace7dU, 
    0x184adf63U, 0x82311ae5U, 0x60335197U, 0x457f5362U, 
    0xe07764b1U, 0x84ae6bbbU, 0x1ca081feU, 0x942b08f9U, 
    0x58684870U, 0x19fd458fU, 0x876cde94U, 0xb7f87b52U, 
    0x23d373abU, 0xe2024b72U, 0x578f1fe3U, 0x2aab5566U, 
    0x0728ebb2U, 0x03c2b52fU, 0x9a7bc586U, 0xa50837d3U, 
    0xf2872830U, 0xb2a5bf23U, 0xba6a0302U, 0x5c8216edU, 
    0x2b1ccf8aU, 0x92b479a7U, 0xf0f207f3U, 0xa1e2694eU, 
    0xcdf4da65U, 0xd5be0506U, 0x1f6234d1U, 0x8afea6c4U, 
    0x9d532e34U, 0xa055f3a2U, 0x32e18a05U, 0x75ebf6a4U, 
    0x39ec830bU, 0xaaef6040U, 0x069f715eU, 0x51106ebdU, 
    0xf98a213eU, 0x3d06dd96U, 0xae053eddU, 0x46bde64dU, 
    0xb58d5491U, 0x055dc471U, 0x6fd40604U, 0xff155060U, 
    0x24fb9819U, 0x97e9bdd6U, 0xcc434089U, 0x779ed967U, 
    0xbd42e8b0U, 0x888b8907U, 0x385b19e7U, 0xdbeec879U, 
    0x470a7ca1U, 0xe90f427cU, 0xc91e84f8U, 0x00000000U, 
    0x83868009U, 0x48ed2b32U, 0xac70111eU, 0x4e725a6cU, 
    0xfbff0efdU, 0x5638850fU, 0x1ed5ae3dU, 0x27392d36U, 
    0x64d90f0aU, 0x21a65c68U, 0xd1545b9bU, 0x3a2e3624U, 
    0xb1670a0cU, 0x0fe75793U, 0xd296eeb4U, 0x9e919b1bU, 
    0x4fc5c080U, 0xa220dc61U, 0x694b775aU, 0x161a121cU, 
    0x0aba93e2U, 0xe52aa0c0U, 0x43e0223cU, 0x1d171b12U, 
    0x0b0d090eU, 0xadc78bf2U, 0xb9a8b62dU, 0xc8a91e14U, 
    0x8519f157U, 0x4c0775afU, 0xbbdd99eeU, 0xfd607fa3U, 
    0x9f2601f7U, 0xbcf5725cU, 0xc53b6644U, 0x347efb5bU, 
    0x7629438bU, 0xdcc623cbU, 0x68fcedb6U, 0x63f1e4b8U, 
    0xcadc31d7U, 0x10856342U, 0x40229713U, 0x2011c684U, 
    0x7d244a85U, 0xf83dbbd2U, 0x1132f9aeU, 0x6da129c7U, 
    0x4b2f9e1dU, 0xf330b2dcU, 0xec52860dU, 0xd0e3c177U, 
    0x6c16b32bU, 0x99b970a9U, 0xfa489411U, 0x2264e947U, 
    0xc48cfca8U, 0x1a3ff0a0U, 0xd82c7d56U, 0xef903322U, 
    0xc74e4987U, 0xc1d138d9U, 0xfea2ca8cU, 0x360bd498U, 
    0xcf81f5a6U, 0x28de7aa5U, 0x268eb7daU, 0xa4bfad3fU, 
    0xe49d3a2cU, 0x0d927850U, 0x9bcc5f6aU, 0x62467e54U, 
    0xc2138df6U, 0xe8b8d890U, 0x5ef7392eU, 0xf5afc382U, 
    0xbe805d9fU, 0x7c93d069U, 0xa92dd56fU, 0xb31225cfU, 
    0x3b99acc8U, 0xa77d1810U, 0x6e639ce8U, 0x7bbb3bdbU, 
    0x097826cdU, 0xf418596eU, 0x01b79aecU, 0xa89a4f83U, 
    0x656e95e6U, 0x7ee6ffaaU, 0x08cfbc21U, 0xe6e815efU, 
    0xd99be7baU, 0xce366f4aU, 0xd4099feaU, 0xd67cb029U, 
    0xafb2a431U, 0x31233f2aU, 0x3094a5c6U, 0xc066a235U, 
    0x37bc4e74U, 0xa6ca82fcU, 0xb0d090e0U, 0x15d8a733U, 
    0x4a9804f1U, 0xf7daec41U, 0x0e50cd7fU, 0x2ff69117U, 
    0x8dd64d76U, 0x4db0ef43U, 0x544daaccU, 0xdf0496e4U, 
    0xe3b5d19eU, 0x1b886a4cU, 0xb81f2cc1U, 0x7f516546U, 
    0x04ea5e9dU, 0x5d358c01U, 0x737487faU, 0x2e410bfbU, 
    0x5a1d67b3U, 0x52d2db92U, 0x335610e9U, 0x1347d66dU, 
    0x8c61d79aU, 0x7a0ca137U, 0x8e14f859U, 0x893c13ebU, 
    0xee27a9ceU, 0x35c961b7U, 0xede51ce1U, 0x3cb1477aU, 
    0x59dfd29cU, 0x3f73f255U, 0x79ce1418U, 0xbf37c773U, 
    0xeacdf753U, 0x5baafd5fU, 0x146f3ddfU, 0x86db4478U, 
    0x81f3afcaU, 0x3ec468b9U, 0x2c342438U, 0x5f40a3c2U, 
    0x72c31d16U, 0x0c25e2bcU, 0x8b493c28U, 0x41950dffU, 
    0x7101a839U, 0xdeb30c08U, 0x9ce4b4d8U, 0x90c15664U, 
    0x6184cb7bU, 0x70b632d5U, 0x745c6c48U, 0x4257b8d0U
};

const tulong aesTd1[] = {
    0xa7f45150U, 0x65417e53U, 0xa4171ac3U, 0x5e273a96U, 
    0x6bab3bcbU, 0x459d1ff1U, 0x58faacabU, 0x03e34b93U, 
    0xfa302055U, 0x6d76adf6U, 0x76cc8891U, 0x4c02f525U, 
    0xd7e54ffcU, 0xcb2ac5d7U, 0x44352680U, 0xa362b58fU, 
    0x5ab1de49U, 0x1bba2567U, 0x0eea4598U, 0xc0fe5de1U, 
    0x752fc302U, 0xf04c8112U, 0x97468da3U, 0xf9d36bc6U, 
    0x5f8f03e7U, 0x9c921595U, 0x7a6dbfebU, 0x595295daU, 
    0x83bed42dU, 0x217458d3U, 0x69e04929U, 0xc8c98e44U, 
    0x89c2756aU, 0x798ef478U, 0x3e58996bU, 0x71b927ddU, 
    0x4fe1beb6U, 0xad88f017U, 0xac20c966U, 0x3ace7db4U, 
    0x4adf6318U, 0x311ae582U, 0x33519760U, 0x7f536245U, 
    0x7764b1e0U, 0xae6bbb84U, 0xa081fe1cU, 0x2b08f994U, 
    0x68487058U, 0xfd458f19U, 0x6cde9487U, 0xf87b52b7U, 
    0xd373ab23U, 0x024b72e2U, 0x8f1fe357U, 0xab55662aU, 
    0x28ebb207U, 0xc2b52f03U, 0x7bc5869aU, 0x0837d3a5U, 
    0x872830f2U, 0xa5bf23b2U, 0x6a0302baU, 0x8216ed5cU, 
    0x1ccf8a2bU, 0xb479a792U, 0xf207f3f0U, 0xe2694ea1U, 
    0xf4da65cdU, 0xbe0506d5U, 0x6234d11fU, 0xfea6c48aU, 
    0x532e349dU, 0x55f3a2a0U, 0xe18a0532U, 0xebf6a475U, 
    0xec830b39U, 0xef6040aaU, 0x9f715e06U, 0x106ebd51U, 
    0x8a213ef9U, 0x06dd963dU, 0x053eddaeU, 0xbde64d46U, 
    0x8d5491b5U, 0x5dc47105U, 0xd406046fU, 0x155060ffU, 
    0xfb981924U, 0xe9bdd697U, 0x434089ccU, 0x9ed96777U, 
    0x42e8b0bdU, 0x8b890788U, 0x5b19e738U, 0xeec879dbU, 
    0x0a7ca147U, 0x0f427ce9U, 0x1e84f8c9U, 0x00000000U, 
    0x86800983U, 0xed2b3248U, 0x70111eacU, 0x725a6c4eU, 
    0xff0efdfbU, 0x38850f56U, 0xd5ae3d1eU, 0x392d3627U, 
    0xd90f0a64U, 0xa65c6821U, 0x545b9bd1U, 0x2e36243aU, 
    0x670a0cb1U, 0xe757930fU, 0x96eeb4d2U, 0x919b1b9eU, 
    0xc5c0804fU, 0x20dc61a2U, 0x4b775a69U, 0x1a121c16U, 
    0xba93e20aU, 0x2aa0c0e5U, 0xe0223c43U, 0x171b121dU, 
    0x0d090e0bU, 0xc78bf2adU, 0xa8b62db9U, 0xa91e14c8U, 
    0x19f15785U, 0x0775af4cU, 0xdd99eebbU, 0x607fa3fdU, 
    0x2601f79fU, 0xf5725cbcU, 0x3b6644c5U, 0x7efb5b34U, 
    0x29438b76U, 0xc623cbdcU, 0xfcedb668U, 0xf1e4b863U, 
    0xdc31d7caU, 0x85634210U, 0x22971340U, 0x11c68420U, 
    0x244a857dU, 0x3dbbd2f8U, 0x32f9ae11U, 0xa129c76dU, 
    0x2f9e1d4bU, 0x30b2dcf3U, 0x52860decU, 0xe3c177d0U, 
    0x16b32b6cU, 0xb970a999U, 0x489411faU, 0x64e94722U, 
    0x8cfca8c4U, 0x3ff0a01aU, 0x2c7d56d8U, 0x903322efU, 
    0x4e4987c7U, 0xd138d9c1U, 0xa2ca8cfeU, 0x0bd49836U, 
    0x81f5a6cfU, 0xde7aa528U, 0x8eb7da26U, 0xbfad3fa4U, 
    0x9d3a2ce4U, 0x9278500dU, 0xcc5f6a9bU, 0x467e5462U, 
    0x138df6c2U, 0xb8d890e8U, 0xf7392e5eU, 0xafc382f5U, 
    0x805d9fbeU, 0x93d0697cU, 0x2dd56fa9U, 0x1225cfb3U, 
    0x99acc83bU, 0x7d1810a7U, 0x639ce86eU, 0xbb3bdb7bU, 
    0x7826cd09U, 0x18596ef4U, 0xb79aec01U, 0x9a4f83a8U, 
    0x6e95e665U, 0xe6ffaa7eU, 0xcfbc2108U, 0xe815efe6U, 
    0x9be7bad9U, 0x366f4aceU, 0x099fead4U, 0x7cb029d6U, 
    0xb2a431afU, 0x233f2a31U, 0x94a5c630U, 0x66a235c0U, 
    0xbc4e7437U, 0xca82fca6U, 0xd090e0b0U, 0xd8a73315U, 
    0x9804f14aU, 0xdaec41f7U, 0x50cd7f0eU, 0xf691172fU, 
    0xd64d768dU, 0xb0ef434dU, 0x4daacc54U, 0x0496e4dfU, 
    0xb5d19ee3U, 0x886a4c1bU, 0x1f2cc1b8U, 0x5165467fU, 
    0xea5e9d04U, 0x358c015dU, 0x7487fa73U, 0x410bfb2eU, 
    0x1d67b35aU, 0xd2db9252U, 0x5610e933U, 0x47d66d13U, 
    0x61d79a8cU, 0x0ca1377aU, 0x14f8598eU, 0x3c13eb89U, 
    0x27a9ceeeU, 0xc961b735U, 0xe51ce1edU, 0xb1477a3cU, 
    0xdfd29c59U, 0x73f2553fU, 0xce141879U, 0x37c773bfU, 
    0xcdf753eaU, 0xaafd5f5bU, 0x6f3ddf14U, 0xdb447886U, 
    0xf3afca81U, 0xc468b93eU, 0x3424382cU, 0x40a3c25fU, 
    0xc31d1672U, 0x25e2bc0cU, 0x493c288bU, 0x950dff41U, 
    0x01a83971U, 0xb30c08deU, 0xe4b4d89cU, 0xc1566490U, 
    0x84cb7b61U, 0xb632d570U, 0x5c6c4874U, 0x57b8d042U
};

const tulong aesTd2[] = {
    0xf45150a7U, 0x417e5365U, 0x171ac3a4U, 0x273a965eU, 
    0xab3bcb6bU, 0x9d1ff145U, 0xfaacab58U, 0xe34b9303U, 
    0x302055faU, 0x76adf66dU, 0xcc889176U, 0x02f5254cU, 
    0xe54ffcd7U, 0x2ac5d7cbU, 0x35268044U, 0x62b58fa3U, 
    0xb1de495aU, 0xba25671bU, 0xea45980eU, 0xfe5de1c0U, 
    0x2fc30275U, 0x4c8112f0U, 0x468da397U, 0xd36bc6f9U, 
    0x8f03e75fU, 0x9215959cU, 0x6dbfeb7aU, 0x5295da59U, 
    0xbed42d83U, 0x7458d321U, 0xe0492969U, 0xc98e44c8U, 
    0xc2756a89U, 0x8ef47879U, 0x58996b3eU, 0xb927dd71U, 
    0xe1beb64fU, 0x88f017adU, 0x20c966acU, 0xce7db43aU, 
    0xdf63184aU, 0x1ae58231U, 0x51976033U, 0x5362457fU, 
    0x64b1e077U, 0x6bbb84aeU, 0x81fe1ca0U, 0x08f9942bU, 
    0x48705868U, 0x458f19fdU, 0xde94876cU, 0x7b52b7f8U, 
    0x73ab23d3U, 0x4b72e202U, 0x1fe3578fU, 0x55662aabU, 
    0xebb20728U, 0xb52f03c2U, 0xc5869a7bU, 0x37d3a508U, 
    0x2830f287U, 0xbf23b2a5U, 0x0302ba6aU, 0x16ed5c82U, 
    0xcf8a2b1cU, 0x79a792b4U, 0x07f3f0f2U, 0x694ea1e2U, 
    0xda65cdf4U, 0x0506d5beU, 0x34d11f62U, 0xa6c48afeU, 
    0x2e349d53U, 0xf3a2a055U, 0x8a0532e1U, 0xf6a475ebU, 
    0x830b39ecU, 0x6040aaefU, 0x715e069fU, 0x6ebd5110U, 
    0x213ef98aU, 0xdd963d06U, 0x3eddae05U, 0xe64d46bdU, 
    0x5491b58dU, 0xc471055dU, 0x06046fd4U, 0x5060ff15U, 
    0x981924fbU, 0xbdd697e9U, 0x4089cc43U, 0xd967779eU, 
    0xe8b0bd42U, 0x8907888bU, 0x19e7385bU, 0xc879dbeeU, 
    0x7ca1470aU, 0x427ce90fU, 0x84f8c91eU, 0x00000000U, 
    0x80098386U, 0x2b3248edU, 0x111eac70U, 0x5a6c4e72U, 
    0x0efdfbffU, 0x850f5638U, 0xae3d1ed5U, 0x2d362739U, 
    0x0f0a64d9U, 0x5c6821a6U, 0x5b9bd154U, 0x36243a2eU, 
    0x0a0cb167U, 0x57930fe7U, 0xeeb4d296U, 0x9b1b9e91U, 
    0xc0804fc5U, 0xdc61a220U, 0x775a694bU, 0x121c161aU, 
    0x93e20abaU, 0xa0c0e52aU, 0x223c43e0U, 0x1b121d17U, 
    0x090e0b0dU, 0x8bf2adc7U, 0xb62db9a8U, 0x1e14c8a9U, 
    0xf1578519U, 0x75af4c07U, 0x99eebbddU, 0x7fa3fd60U, 
    0x01f79f26U, 0x725cbcf5U, 0x6644c53bU, 0xfb5b347eU, 
    0x438b7629U, 0x23cbdcc6U, 0xedb668fcU, 0xe4b863f1U, 
    0x31d7cadcU, 0x63421085U, 0x97134022U, 0xc6842011U, 
    0x4a857d24U, 0xbbd2f83dU, 0xf9ae1132U, 0x29c76da1U, 
    0x9e1d4b2fU, 0xb2dcf330U, 0x860dec52U, 0xc177d0e3U, 
    0xb32b6c16U, 0x70a999b9U, 0x9411fa48U, 0xe9472264U, 
    0xfca8c48cU, 0xf0a01a3fU, 0x7d56d82cU, 0x3322ef90U, 
    0x4987c74eU, 0x38d9c1d1U, 0xca8cfea2U, 0xd498360bU, 
    0xf5a6cf81U, 0x7aa528deU, 0xb7da268eU, 0xad3fa4bfU, 
    0x3a2ce49dU, 0x78500d92U, 0x5f6a9bccU, 0x7e546246U, 
    0x8df6c213U, 0xd890e8b8U, 0x392e5ef7U, 0xc382f5afU, 
    0x5d9fbe80U, 0xd0697c93U, 0xd56fa92dU, 0x25cfb312U, 
    0xacc83b99U, 0x1810a77dU, 0x9ce86e63U, 0x3bdb7bbbU, 
    0x26cd0978U, 0x596ef418U, 0x9aec01b7U, 0x4f83a89aU, 
    0x95e6656eU, 0xffaa7ee6U, 0xbc2108cfU, 0x15efe6e8U, 
    0xe7bad99bU, 0x6f4ace36U, 0x9fead409U, 0xb029d67cU, 
    0xa431afb2U, 0x3f2a3123U, 0xa5c63094U, 0xa235c066U, 
    0x4e7437bcU, 0x82fca6caU, 0x90e0b0d0U, 0xa73315d8U, 
    0x04f14a98U, 0xec41f7daU, 0xcd7f0e50U, 0x91172ff6U, 
    0x4d768dd6U, 0xef434db0U, 0xaacc544dU, 0x96e4df04U, 
    0xd19ee3b5U, 0x6a4c1b88U, 0x2cc1b81fU, 0x65467f51U, 
    0x5e9d04eaU, 0x8c015d35U, 0x87fa7374U, 0x0bfb2e41U, 
    0x67b35a1dU, 0xdb9252d2U, 0x10e93356U, 0xd66d1347U, 
    0xd79a8c61U, 0xa1377a0cU, 0xf8598e14U, 0x13eb893cU, 
    0xa9ceee27U, 0x61b735c9U, 0x1ce1ede5U, 0x477a3cb1U, 
    0xd29c59dfU, 0xf2553f73U, 0x141879ceU, 0xc773bf37U, 
    0xf753eacdU, 0xfd5f5baaU, 0x3ddf146fU, 0x447886dbU, 
    0xafca81f3U, 0x68b93ec4U, 0x24382c34U, 0xa3c25f40U, 
    0x1d1672c3U, 0xe2bc0c25U, 0x3c288b49U, 0x0dff4195U, 
    0xa8397101U, 0x0c08deb3U, 0xb4d89ce4U, 0x566490c1U, 
    0xcb7b6184U, 0x32d570b6U, 0x6c48745cU, 0xb8d04257U
};

const tulong aesTd3[] = {
    0x5150a7f4U, 0x7e536541U, 0x1ac3a417U, 0x3a965e27U, 
    0x3bcb6babU, 0x1ff1459dU, 0xacab58faU, 0x4b9303e3U, 
    0x2055fa30U, 0xadf66d76U, 0x889176ccU, 0xf5254c02U, 
    0x4ffcd7e5U, 0xc5d7cb2aU, 0x26804435U, 0xb58fa362U, 
    0xde495ab1U, 0x25671bbaU, 0x45980eeaU, 0x5de1c0feU, 
    0xc302752fU, 0x8112f04cU, 0x8da39746U, 0x6bc6f9d3U, 
    0x03e75f8fU, 0x15959c92U, 0xbfeb7a6dU, 0x95da5952U, 
    0xd42d83beU, 0x58d32174U, 0x492969e0U, 0x8e44c8c9U, 
    0x756a89c2U, 0xf478798eU, 0x996b3e58U, 0x27dd71b9U, 
    0xbeb64fe1U, 0xf017ad88U, 0xc966ac20U, 0x7db43aceU, 
    0x63184adfU, 0xe582311aU, 0x97603351U, 0x62457f53U, 
    0xb1e07764U, 0xbb84ae6bU, 0xfe1ca081U, 0xf9942b08U, 
    0x70586848U, 0x8f19fd45U, 0x94876cdeU, 0x52b7f87bU, 
    0xab23d373U, 0x72e2024bU, 0xe3578f1fU, 0x662aab55U, 
    0xb20728ebU, 0x2f03c2b5U, 0x869a7bc5U, 0xd3a50837U, 
    0x30f28728U, 0x23b2a5bfU, 0x02ba6a03U, 0xed5c8216U, 
    0x8a2b1ccfU, 0xa792b479U, 0xf3f0f207U, 0x4ea1e269U, 
    0x65cdf4daU, 0x06d5be05U, 0xd11f6234U, 0xc48afea6U, 
    0x349d532eU, 0xa2a055f3U, 0x0532e18aU, 0xa475ebf6U, 
    0x0b39ec83U, 0x40aaef60U, 0x5e069f71U, 0xbd51106eU, 
    0x3ef98a21U, 0x963d06ddU, 0xddae053eU, 0x4d46bde6U, 
    0x91b58d54U, 0x71055dc4U, 0x046fd406U, 0x60ff1550U, 
    0x1924fb98U, 0xd697e9bdU, 0x89cc4340U, 0x67779ed9U, 
    0xb0bd42e8U, 0x07888b89U, 0xe7385b19U, 0x79dbeec8U, 
    0xa1470a7cU, 0x7ce90f42U, 0xf8c91e84U, 0x00000000U, 
    0x09838680U, 0x3248ed2bU, 0x1eac7011U, 0x6c4e725aU, 
    0xfdfbff0eU, 0x0f563885U, 0x3d1ed5aeU, 0x3627392dU, 
    0x0a64d90fU, 0x6821a65cU, 0x9bd1545bU, 0x243a2e36U, 
    0x0cb1670aU, 0x930fe757U, 0xb4d296eeU, 0x1b9e919bU, 
    0x804fc5c0U, 0x61a220dcU, 0x5a694b77U, 0x1c161a12U, 
    0xe20aba93U, 0xc0e52aa0U, 0x3c43e022U, 0x121d171bU, 
    0x0e0b0d09U, 0xf2adc78bU, 0x2db9a8b6U, 0x14c8a91eU, 
    0x578519f1U, 0xaf4c0775U, 0xeebbdd99U, 0xa3fd607fU, 
    0xf79f2601U, 0x5cbcf572U, 0x44c53b66U, 0x5b347efbU, 
    0x8b762943U, 0xcbdcc623U, 0xb668fcedU, 0xb863f1e4U, 
    0xd7cadc31U, 0x42108563U, 0x13402297U, 0x842011c6U, 
    0x857d244aU, 0xd2f83dbbU, 0xae1132f9U, 0xc76da129U, 
    0x1d4b2f9eU, 0xdcf330b2U, 0x0dec5286U, 0x77d0e3c1U, 
    0x2b6c16b3U, 0xa999b970U, 0x11fa4894U, 0x472264e9U, 
    0xa8c48cfcU, 0xa01a3ff0U, 0x56d82c7dU, 0x22ef9033U, 
    0x87c74e49U, 0xd9c1d138U, 0x8cfea2caU, 0x98360bd4U, 
    0xa6cf81f5U, 0xa528de7aU, 0xda268eb7U, 0x3fa4bfadU, 
    0x2ce49d3aU, 0x500d9278U, 0x6a9bcc5fU, 0x5462467eU, 
    0xf6c2138dU, 0x90e8b8d8U, 0x2e5ef739U, 0x82f5afc3U, 
    0x9fbe805dU, 0x697c93d0U, 0x6fa92dd5U, 0xcfb31225U, 
    0xc83b99acU, 0x10a77d18U, 0xe86e639cU, 0xdb7bbb3bU, 
    0xcd097826U, 0x6ef41859U, 0xec01b79aU, 0x83a89a4fU, 
    0xe6656e95U, 0xaa7ee6ffU, 0x2108cfbcU, 0xefe6e815U, 
    0xbad99be7U, 0x4ace366fU, 0xead4099fU, 0x29d67cb0U, 
    0x31afb2a4U, 0x2a31233fU, 0xc63094a5U, 0x35c066a2U, 
    0x7437bc4eU, 0xfca6ca82U, 0xe0b0d090U, 0x3315d8a7U, 
    0xf14a9804U, 0x41f7daecU, 0x7f0e50cdU, 0x172ff691U, 
    0x768dd64dU, 0x434db0efU, 0xcc544daaU, 0xe4df0496U, 
    0x9ee3b5d1U, 0x4c1b886aU, 0xc1b81f2cU, 0x467f5165U, 
    0x9d04ea5eU, 0x015d358cU, 0xfa737487U, 0xfb2e410bU, 
    0xb35a1d67U, 0x9252d2dbU, 0xe9335610U, 0x6d1347d6U, 
    0x9a8c61d7U, 0x377a0ca1U, 0x598e14f8U, 0xeb893c13U, 
    0xceee27a9U, 0xb735c961U, 0xe1ede51cU, 0x7a3cb147U, 
    0x9c59dfd2U, 0x553f73f2U, 0x1879ce14U, 0x73bf37c7U, 
    0x53eacdf7U, 0x5f5baafdU, 0xdf146f3dU, 0x7886db44U, 
    0xca81f3afU, 0xb93ec468U, 0x382c3424U, 0xc25f40a3U, 
    0x1672c31dU, 0xbc0c25e2U, 0x288b493cU, 0xff41950dU, 
    0x397101a8U, 0x08deb30cU, 0xd89ce4b4U, 0x6490c156U, 
    0x7b6184cbU, 0xd570b632U, 0x48745c6cU, 0xd04257b8U
};

const tulong aesTd4[] = {
0x52525252U, 0x09090909U, 0x6a6a6a6aU, 0xd5d5d5d5U, 
    0x30303030U, 0x36363636U, 0xa5a5a5a5U, 0x38383838U, 
    0xbfbfbfbfU, 0x40404040U, 0xa3a3a3a3U, 0x9e9e9e9eU, 
    0x81818181U, 0xf3f3f3f3U, 0xd7d7d7d7U, 0xfbfbfbfbU, 
    0x7c7c7c7cU, 0xe3e3e3e3U, 0x39393939U, 0x82828282U, 
    0x9b9b9b9bU, 0x2f2f2f2fU, 0xffffffffU, 0x87878787U, 
    0x34343434U, 0x8e8e8e8eU, 0x43434343U, 0x44444444U, 
    0xc4c4c4c4U, 0xdedededeU, 0xe9e9e9e9U, 0xcbcbcbcbU, 
    0x54545454U, 0x7b7b7b7bU, 0x94949494U, 0x32323232U, 
    0xa6a6a6a6U, 0xc2c2c2c2U, 0x23232323U, 0x3d3d3d3dU, 
    0xeeeeeeeeU, 0x4c4c4c4cU, 0x95959595U, 0x0b0b0b0bU, 
    0x42424242U, 0xfafafafaU, 0xc3c3c3c3U, 0x4e4e4e4eU, 
    0x08080808U, 0x2e2e2e2eU, 0xa1a1a1a1U, 0x66666666U, 
    0x28282828U, 0xd9d9d9d9U, 0x24242424U, 0xb2b2b2b2U, 
    0x76767676U, 0x5b5b5b5bU, 0xa2a2a2a2U, 0x49494949U, 
    0x6d6d6d6dU, 0x8b8b8b8bU, 0xd1d1d1d1U, 0x25252525U, 
    0x72727272U, 0xf8f8f8f8U, 0xf6f6f6f6U, 0x64646464U, 
    0x86868686U, 0x68686868U, 0x98989898U, 0x16161616U, 
    0xd4d4d4d4U, 0xa4a4a4a4U, 0x5c5c5c5cU, 0xccccccccU, 
    0x5d5d5d5dU, 0x65656565U, 0xb6b6b6b6U, 0x92929292U, 
    0x6c6c6c6cU, 0x70707070U, 0x48484848U, 0x50505050U, 
    0xfdfdfdfdU, 0xededededU, 0xb9b9b9b9U, 0xdadadadaU, 
    0x5e5e5e5eU, 0x15151515U, 0x46464646U, 0x57575757U, 
    0xa7a7a7a7U, 0x8d8d8d8dU, 0x9d9d9d9dU, 0x84848484U, 
    0x90909090U, 0xd8d8d8d8U, 0xababababU, 0x00000000U, 
    0x8c8c8c8cU, 0xbcbcbcbcU, 0xd3d3d3d3U, 0x0a0a0a0aU, 
    0xf7f7f7f7U, 0xe4e4e4e4U, 0x58585858U, 0x05050505U, 
    0xb8b8b8b8U, 0xb3b3b3b3U, 0x45454545U, 0x06060606U, 
    0xd0d0d0d0U, 0x2c2c2c2cU, 0x1e1e1e1eU, 0x8f8f8f8fU, 
    0xcacacacaU, 0x3f3f3f3fU, 0x0f0f0f0fU, 0x02020202U, 
    0xc1c1c1c1U, 0xafafafafU, 0xbdbdbdbdU, 0x03030303U, 
    0x01010101U, 0x13131313U, 0x8a8a8a8aU, 0x6b6b6b6bU, 
    0x3a3a3a3aU, 0x91919191U, 0x11111111U, 0x41414141U, 
    0x4f4f4f4fU, 0x67676767U, 0xdcdcdcdcU, 0xeaeaeaeaU, 
    0x97979797U, 0xf2f2f2f2U, 0xcfcfcfcfU, 0xcecececeU, 
    0xf0f0f0f0U, 0xb4b4b4b4U, 0xe6e6e6e6U, 0x73737373U, 
    0x96969696U, 0xacacacacU, 0x74747474U, 0x22222222U, 
    0xe7e7e7e7U, 0xadadadadU, 0x35353535U, 0x85858585U, 
    0xe2e2e2e2U, 0xf9f9f9f9U, 0x37373737U, 0xe8e8e8e8U, 
    0x1c1c1c1cU, 0x75757575U, 0xdfdfdfdfU, 0x6e6e6e6eU, 
    0x47474747U, 0xf1f1f1f1U, 0x1a1a1a1aU, 0x71717171U, 
    0x1d1d1d1dU, 0x29292929U, 0xc5c5c5c5U, 0x89898989U, 
    0x6f6f6f6fU, 0xb7b7b7b7U, 0x62626262U, 0x0e0e0e0eU, 
    0xaaaaaaaaU, 0x18181818U, 0xbebebebeU, 0x1b1b1b1bU, 
    0xfcfcfcfcU, 0x56565656U, 0x3e3e3e3eU, 0x4b4b4b4bU, 
    0xc6c6c6c6U, 0xd2d2d2d2U, 0x79797979U, 0x20202020U, 
    0x9a9a9a9aU, 0xdbdbdbdbU, 0xc0c0c0c0U, 0xfefefefeU, 
    0x78787878U, 0xcdcdcdcdU, 0x5a5a5a5aU, 0xf4f4f4f4U, 
    0x1f1f1f1fU, 0xddddddddU, 0xa8a8a8a8U, 0x33333333U, 
    0x88888888U, 0x07070707U, 0xc7c7c7c7U, 0x31313131U, 
    0xb1b1b1b1U, 0x12121212U, 0x10101010U, 0x59595959U, 
    0x27272727U, 0x80808080U, 0xececececU, 0x5f5f5f5fU, 
    0x60606060U, 0x51515151U, 0x7f7f7f7fU, 0xa9a9a9a9U, 
    0x19191919U, 0xb5b5b5b5U, 0x4a4a4a4aU, 0x0d0d0d0dU, 
    0x2d2d2d2dU, 0xe5e5e5e5U, 0x7a7a7a7aU, 0x9f9f9f9fU, 
    0x93939393U, 0xc9c9c9c9U, 0x9c9c9c9cU, 0xefefefefU, 
    0xa0a0a0a0U, 0xe0e0e0e0U, 0x3b3b3b3bU, 0x4d4d4d4dU, 
    0xaeaeaeaeU, 0x2a2a2a2aU, 0xf5f5f5f5U, 0xb0b0b0b0U, 
    0xc8c8c8c8U, 0xebebebebU, 0xbbbbbbbbU, 0x3c3c3c3cU, 
    0x83838383U, 0x53535353U, 0x99999999U, 0x61616161U, 
    0x17171717U, 0x2b2b2b2bU, 0x04040404U, 0x7e7e7e7eU, 
    0xbabababaU, 0x77777777U, 0xd6d6d6d6U, 0x26262626U, 
    0xe1e1e1e1U, 0x69696969U, 0x14141414U, 0x63636363U, 
    0x55555555U, 0x21212121U, 0x0c0c0c0cU, 0x7d7d7d7dU
};

extern const tulong aesTe4[];

/******************************************************************************
  * FUNCTION: aesDecrypt    
  * PURPOSE: AES decryption algorithm
  ******************************************************************************/
void aesDecrypt(tulong * cga_RESTRICT rk, tword ct[cga_RESTRICT], tword pt[cga_RESTRICT], tint nr) 
{
  const v128_t * cga_RESTRICT round_key;
  int i;
  double v8_76543210, v8_fedcba98;
  double v8_pltx0, v8_pltx1;
  tulong * cga_RESTRICT Td0_le = (tulong *)&aesTd0;
  tulong * cga_RESTRICT Td1_le = (tulong *)&aesTd1;
  tulong * cga_RESTRICT Td2_le = (tulong *)&aesTd2;
  tulong * cga_RESTRICT Td3_le = (tulong *)&aesTd3;
  tulong * cga_RESTRICT Td4_le = (tulong *)&aesTd4;

  /* add in the subkey */
  v8_76543210 = _memd8((void*)&ct[0]);
  v8_fedcba98 = _memd8((void*)&ct[8]);
  v8_pltx0    = _memd8((void*)&rk[0]);
  v8_pltx1    = _memd8((void*)&rk[2]);
  v8_76543210 = _itod(_hi(v8_76543210)^_hi(v8_pltx0),_lo(v8_76543210)^_lo(v8_pltx0));
  v8_fedcba98 = _itod(_hi(v8_fedcba98)^_hi(v8_pltx1),_lo(v8_fedcba98)^_lo(v8_pltx1));

  /* now do nine rounds */
  cga_LOOP_COUNT_INFO(9,13,1,9,9)
  for(i=1;i<nr;i++) {
	round_key = (v128_t *)&rk[i*4];
	{
	  uint32_t column0, column1, column2, column3;
	  double round_key_v32_0, round_key_v32_2;

	  uint32_t index_fedc, index_ba98, index_7654, index_3210;
	  const uint32_t * cga_RESTRICT ptr = (uint32_t*)round_key->v32;

	  index_3210      = _lo(v8_76543210);
	  index_7654      = _hi(v8_76543210);	
	  index_ba98      = _lo(v8_fedcba98);
	  index_fedc      = _hi(v8_fedcba98);

	  column0         = Td0_le[_extu(index_3210,24,24)] ^ Td1_le[_extu(index_fedc,16,24)] ^ Td2_le[_extu(index_ba98,8,24)] ^ Td3_le[_extu(index_7654,0,24)];
	  column1         = Td0_le[_extu(index_7654,24,24)] ^ Td1_le[_extu(index_3210,16,24)] ^ Td2_le[_extu(index_fedc,8,24)] ^ Td3_le[_extu(index_ba98,0,24)];
	  column2         = Td0_le[_extu(index_ba98,24,24)] ^ Td1_le[_extu(index_7654,16,24)] ^ Td2_le[_extu(index_3210,8,24)] ^ Td3_le[_extu(index_fedc,0,24)];
	  column3         = Td0_le[_extu(index_fedc,24,24)] ^ Td1_le[_extu(index_ba98,16,24)] ^ Td2_le[_extu(index_7654,8,24)] ^ Td3_le[_extu(index_3210,0,24)];

	  round_key_v32_0 = _amemd8((void*)&ptr[0]);
	  round_key_v32_2 = _amemd8((void*)&ptr[2]);
	  v8_76543210     = _itod(column1^_hi(round_key_v32_0),column0^_lo(round_key_v32_0));
	  v8_fedcba98     = _itod(column3^_hi(round_key_v32_2),column2^_lo(round_key_v32_2));
	}
  }

  /* the last round is different */
  round_key = (v128_t *)&rk[nr*4];
  {
    uint32_t index_fedc, index_ba98, index_7654, index_3210;
    uint32_t state_fedc, state_ba98, state_7654, state_3210;
    double v8_pltx0, v8_pltx1;

    index_3210  = _lo(v8_76543210);	index_7654  = _hi(v8_76543210);	
    index_ba98  = _lo(v8_fedcba98);	index_fedc  = _hi(v8_fedcba98);

    // byte substitutions and row shifts 
    state_3210   =   (Td4_le[_extu(index_3210,24,24)] & 0x000000ff)
                   | (Td4_le[_extu(index_fedc,16,24)] & 0x0000ff00)
			       | (Td4_le[_extu(index_ba98, 8,24)] & 0x00ff0000)
			       | (Td4_le[_extu(index_7654, 0,24)] & 0xff000000);
    state_7654   =   (Td4_le[_extu(index_7654,24,24)] & 0x000000ff)
                   | (Td4_le[_extu(index_3210,16,24)] & 0x0000ff00)
			       | (Td4_le[_extu(index_fedc, 8,24)] & 0x00ff0000)
			       | (Td4_le[_extu(index_ba98, 0,24)] & 0xff000000);
    state_ba98   =   (Td4_le[_extu(index_ba98,24,24)] & 0x000000ff)
                   | (Td4_le[_extu(index_7654,16,24)] & 0x0000ff00)
			       | (Td4_le[_extu(index_3210, 8,24)] & 0x00ff0000)
			       | (Td4_le[_extu(index_fedc, 0,24)] & 0xff000000);
    state_fedc   =   (Td4_le[_extu(index_fedc,24,24)] & 0x000000ff)
                   | (Td4_le[_extu(index_ba98,16,24)] & 0x0000ff00)
			       | (Td4_le[_extu(index_7654, 8,24)] & 0x00ff0000)
			       | (Td4_le[_extu(index_3210, 0,24)] & 0xff000000);
  
    v8_pltx0    = _memd8((void*)&round_key->v32[0]);
    v8_pltx1    = _memd8((void*)&round_key->v32[2]);
    _memd8((void*)&pt[0]) = _itod(state_7654^_hi(v8_pltx0),state_3210^_lo(v8_pltx0));
    _memd8((void*)&pt[8]) = _itod(state_fedc^_hi(v8_pltx1),state_ba98^_lo(v8_pltx1));
  }
}

/******************************************************************************
  * FUNCTION: aesKeyExpandDec    
  * PURPOSE: Expand the cipher key into the decryption key schedule
  ******************************************************************************/
tint aesKeyExpandDec(tulong * cga_RESTRICT rk, tword cipherKey[cga_RESTRICT], tint keyBits)
{
    tint Nr, i, j;
    tulong temp;      
    tulong * cga_RESTRICT Td0_le = (tulong *)&aesTd0;
    tulong * cga_RESTRICT Td1_le = (tulong *)&aesTd1;
    tulong * cga_RESTRICT Td2_le = (tulong *)&aesTd2;
    tulong * cga_RESTRICT Td3_le = (tulong *)&aesTd3;
    tulong * cga_RESTRICT Te4_le = (tulong *)&aesTe4;

    _memd8((void*)&rk[0]) =   _memd8((void*)&cipherKey[0]);
    _memd8((void*)&rk[2]) =   _memd8((void*)&cipherKey[8]);

  /*
    * Td0[x] = Si[x].[0e, 09, 0d, 0b];
    * Td1[x] = Si[x].[0b, 0e, 09, 0d];
    * Td2[x] = Si[x].[0d, 0b, 0e, 09];
    * Td3[x] = Si[x].[09, 0d, 0b, 0e];
    * Td4[x] = Si[x].[01, 01, 01, 01];
    */

    /* expand the cipher key: */
    Nr = aesKeyExpandEnc(rk, cipherKey, keyBits);
    /* invert the order of the round keys: */
    cga_LOOP_COUNT_INFO(5,7,1,5,5)
    for (i = 0, j = 4*Nr; i < j; i += 4, j -= 4) {
        temp = rk[i    ]; rk[i    ] = rk[j    ]; rk[j    ] = temp;
        temp = rk[i + 1]; rk[i + 1] = rk[j + 1]; rk[j + 1] = temp;
        temp = rk[i + 2]; rk[i + 2] = rk[j + 2]; rk[j + 2] = temp;
        temp = rk[i + 3]; rk[i + 3] = rk[j + 3]; rk[j + 3] = temp;
    }
    /* apply the inverse MixColumn transform to all round keys but the first and the last: */
    cga_LOOP_COUNT_INFO(10,14,2,10,10)
    for (i = 1; i < Nr; i++) {
        rk += 4;
        rk[0] =
            Td0_le[Te4_le[_extu(rk[0],24,24)] & 0xff] ^
            Td1_le[Te4_le[_extu(rk[0],16,24)] & 0xff] ^
            Td2_le[Te4_le[_extu(rk[0],8,24)] & 0xff] ^
            Td3_le[Te4_le[_extu(rk[0],0,24)] & 0xff];
        rk[1] =
            Td0_le[Te4_le[_extu(rk[1],24,24)] & 0xff] ^
            Td1_le[Te4_le[_extu(rk[1],16,24)] & 0xff] ^
            Td2_le[Te4_le[_extu(rk[1],8,24)] & 0xff] ^
            Td3_le[Te4_le[_extu(rk[1],0,24)] & 0xff];
        rk[2] =
            Td0_le[Te4_le[_extu(rk[2],24,24)] & 0xff] ^
            Td1_le[Te4_le[_extu(rk[2],16,24)] & 0xff] ^
            Td2_le[Te4_le[_extu(rk[2],8,24)] & 0xff] ^
            Td3_le[Te4_le[_extu(rk[2],0,24)] & 0xff];
        rk[3] =
            Td0_le[Te4_le[_extu(rk[3],24,24)] & 0xff] ^
            Td1_le[Te4_le[_extu(rk[3],16,24)] & 0xff] ^
            Td2_le[Te4_le[_extu(rk[3],8,24)] & 0xff] ^
            Td3_le[Te4_le[_extu(rk[3],0,24)] & 0xff];
    }
    return Nr;
}
/* nothing past this point */
