/*
 * This is a little header file which doxygen parses to generate the main
 * documentation page
 */

/**
 * @mainpage SHA1 Package
 *
 * @section description Description
 *
 *  The Secure Hash Algorithm 1, SHA1, is, as specified by RFC 3174, a method for computing
 *  a condensed representation of a message or data file.  When a message of any 
 *  length < 2^64 bits is input, the SHA-1 produces a 160-bit output called a message digest.  
 *  The message digest can then, for example, be input to a signature algorithm which 
 *  generates or verifies the signature for the message.  Signing the message digest
 *  rather than the message often improves the efficiency of the process because the message
 *  digest is usually much smaller in size than the message.  The same hash algorithm must 
 *  be used by the verifier of a digital signature as was used by the creator of the digital
 *  signature.  Any change to the message in transit will, with very high probability, result in 
 *  a different message digest, and the signature will fail to verify.
 *  
 *  The SHA-1 is called secure because it is computationally infeasible to find a message 
 *  which corresponds to a given message digest, or to find two different messages which 
 *  produce the same message digest.  Any change to a message in transit will, with very 
 *  high probability, result in a different message digest, and the signature will fail to verify.
 * 
 *  For more information on the SHA1 algorithm please refer to following references: <BR>
 *  FIPS Secure Hash Standard - http://www.itl.nist.gov/fipspubs/fip180-1.htm <BR>
 *  SHA1 RFC - http://www.ietf.org/rfc/rfc3174.txt <BR>
 *
 * @subsection optimization HMAC-Level Optimization
 *
 *  The SHA1 source in this package provides the necessary hooks in order to implement a
 *  HMAC-level optimization described by RFC2104 - Section 4.  As part of the HMAC
 *  computation two special hashes must be created.  The first hash is a key XOR'd with the
 *  string 0x36 repeated B times.  The second is the same key XOR'd with the string 0x5C
 *  repeated B times.  Where B is normally 64.  These hashes are then used to compute the
 *  HMAC for the given data.  If the key does not change for an arbitrarily long period of time
 *  these specially computed hashes will not change each time they are computed and thus
 *  after the initial pass through the HMAC computation code can be saved for use in later
 *  HMAC computations.  The API of @ref sha1Init provides a pointer which allows the SHA1 
 *  algorithm instance structure to be initialized to one of these saved hashes.  Also, the 
 *  @ref sha1_SAVEKPADHASH macro provides the ability to store a computed hash into a 
 *  designated @ref sha1Inst_t storage structure defined at the HMAC level.
 *
 *  For more information on the HMAC level please refer to the following RFC: <BR>
 *  HMAC RFC - http://www.ietf.org/rfc/rfc2104.txt <BR>
 *
 * @subsection usecases Use Cases
 *
 *  The SHA-1 algorithm is part of a family of secure hash algorithms required by law in certain
 *  U.S. government applications for the protection of sensitive unclassified information.  It's used
 *  within multiple security applications and protocols, including SSL, SSH, and IPsec.  The SHA-1 
 *  algorithm is predominantly used to verify the integrity of a message or password.  It can also 
 *  be used within P2P filesharing networks to locate and identify files.  
 *
 * @subsection licenseinfo License, Standard, and Export Control Information
 *
 *  For the source code licensing, standard, and export control information please see the release notes
 *  provided with this package.
 *
 * @section libraries Target Library Information
 *
 *  Different libaries will be produced depending upon target build.  There are three library types.
 *  Libraries *_a.a* will contain assembly or intrinsic optimized code.  Libraries *_c.a* will contain
 *  C source code which does not have an assembly or intrinsic counterpart.  Libraries *_cm.a*
 *  will contain the C model code of the assembly or intrinsic code contained in the *_a.a* library.
 *  If a target build has a *_a.a* library it will always also have a *_cm.a* in case the component
 *  integrator would like to use the C model code instead of the assembly/intrinsic code.  For a
 *  guideline on which library combinations are required for linking please refer to linker 
 *  command files, *.cmd, located in the @c sha1\\test\\lnkr\\\<target\>\\ directory.
 *
 * @subsection libnotes Library Notes
 *
 *  The following libraries are provided in this package:<BR>
 *
 *  -# sha1_size_c.a64P - Little endian c64x+ COFF library containing code written for minimal size<BR>
 *  -# sha1_speed_c.a64P - Little endian c64x+ COFF library containing code written for minimal cycles<BR>
 *  -# sha1_size_c.a64Pe - Big endian c64x+ COFF library containing code written for minimal size<BR>
 *  -# sha1_speed_c.a64Pe - Big endian c64x+ COFF library containing code written for minimal cycles<BR>
 *  -# sha1_size_c.ae64P - Little endian c64x+ ELF library containing code written for minimal size<BR>
 *  -# sha1_speed_c.ea64P - Little endian c64x+ ELF library containing code written for minimal cycles<BR>
 *  -# sha1_size_c.ae64Pe - Big endian c64x+ ELF library containing code written for minimal size<BR>
 *  -# sha1_speed_c.ae64Pe - Big endian c64x+ ELF library containing code written for minimal cycles<BR>
 *  -# sha1_c.a55L - c55x library containing core SHA-1 functions<BR>
 *  -# sha1_a.a55L - c55x library containing assembly implementation of SHA1 hashing function<BR>
 *  -# sha1_cm.a55L - c55x library containing C model implementation of SHA1 hashing function<BR>
 *  -# sha1_size_c.a11 - Little endian ARM11 library containing code written for minimal size<BR>
 *  -# sha1_speed_c.a11 - Little endian ARM11 library containing code written for minimal cycles<BR>
 *  -# sha1_size_c.a11e - Big endian ARM11 library containing code written for minimal size<BR>
 *  -# sha1_speed_c.a11e - Big endian ARM11 library containing code written for minimal cycles<BR>
 *  -# sha1_size_c.a66 - Little endian c66 COFF library containing code written for minimal size<BR>
 *  -# sha1_speed_c.a66 - Little endian c66 COFF library containing code written for minimal cycles<BR>
 *  -# sha1_size_c.a66e - Big endian c66 COFF library containing code written for minimal size<BR>
 *  -# sha1_speed_c.a66e - Big endian c66 COFF library containing code written for minimal cycles<BR>
 *  -# sha1_size_c.ae66 - Little endian c66 ELF library containing code written for minimal size<BR>
 *  -# sha1_speed_c.ea66 - Little endian c66 ELF library containing code written for minimal cycles<BR>
 *  -# sha1_size_c.ae66e - Big endian c66 ELF library containing code written for minimal size<BR>
 *  -# sha1_speed_c.ae66e - Big endian c66 ELF library containing code written for minimal cycles<BR>
 *  -# sha1_size_c.a674 - Little endian c674 COFF library containing code written for minimal size<BR>
 *  -# sha1_speed_c.a674 - Little endian c674 COFF library containing code written for minimal cycles<BR>
 *  -# sha1_size_c.a674e - Big endian c674 COFF library containing code written for minimal size<BR>
 *  -# sha1_speed_c.a674e - Big endian c674 COFF library containing code written for minimal cycles<BR>
 *  -# sha1_size_c.ae674 - Little endian c674 ELF library containing code written for minimal size<BR>
 *  -# sha1_speed_c.ea674 - Little endian c674 ELF library containing code written for minimal cycles<BR>
 *  -# sha1_size_c.ae674e - Big endian c674 ELF library containing code written for minimal size<BR>
 *  -# sha1_speed_c.ae674e - Big endian c674 ELF library containing code written for minimal cycles<BR>
 
 *
 * @section integration Integration Information
 *  
 *  There is only one module in this package which is the SHA1 module. 
 *  The API documentation is located at @ref SHA1. 
 *  The API header is located at @c sha1\\sha1.h. Target library location is @c sha1\\\<target\>\\sha1_*.a*
 *  There may be multiple libraries (assembly, C, etc.) for release on some targets. For information on 
 *  which libraries are required for linking, refer to @c sha1\\test\\lnkr\\\<target\>\\*.cmd files.
 *  Targets provided in this package may include one or more of c55l (55 large), 
 *  c64le (64 little endian), c64be (64 big endian), c64Ple (64+ little endian) and c64Pbe (64+ big endian),
 *  tiarmv6le (ARM11 little endian), tiarmv6be (ARM11 big endian), c66le (66 little Endian), c66be (66 big endian),
 *  c674le (674 little Endian), c674be (674 big endian),
 * 
 *  These names are standard xdc target names.
 *  For MIPS and Memory Consumption information, refer to documentation in SHA1's test package.
 *
 *  For consumers of this package that do not use xdc tools to build their sources (e.g use GNU make instead),
 *  xdc dependency will need to be emulated when building sources that include the API headers of this release.
 *  The following defines will need to be included when compiling sources:
 *  @verbatim 
      -Dti_targets_<target> -Dxdc_target_name__=<target> -Dxdc_target_types__=ti/targets/std.h -Dxdc_nolocalnames

    where <target> is one of C54, C55, C55_large, C64, C64_big_endian, C64P, C64P_big_endian, C64P_elf, C64P_big_endian_elf,
    C66, C66_big_endian, C66_elf, C66_big_endian_elf, C674, C674_big_endian, C674_elf, C674_big_endian_elf, TIArmv6le, or TIArmv6be, 
    
    which are xdc standard target names (note all targets may not apply to this release) 
    Here C64 means C64 little endian and C64P is c64x+ little endian (P is for + in c64x+)
    @endverbatim
 *  Note @c std.h is needed from xdc tools package even if xdc is not used to build the sources.
 *
 * @section dependencies Dependent Packages
 *  - ti.mas.types (Common data type defintions)
 *  - ti.mas.secutil  (Common security utilities)
 *  - ti.mas.swtools (Internally used s/w tools)
 */

/**
 * @page Revision History
 * <center> <B> Revision History </B> </center>
 *
 * <TABLE align=center>
 * <TR>
 * <TD><B>Version</B></TD>
 * <TD><B>Comments</B></TD>
 * </TR>
 * <TR>
 * <TD>1.1.2.0</TD>
 * <TD>Support for new tools, ELF support</TD>
 * </TR>
 * <TR>
 * <TD>1.1.1.0</TD>
 * <TD>Support for new tools, C66 support</TD>
 * </TR>
 * <TR>
 * <TD>1.1.0.0</TD>
 * <TD>Support for new tools, ARM11 support</TD>
 * </TR>
 * <TR>
 * <TD>1.0.0.3</TD>
 * <TD>Support for new tools, c55x support, package cleanup and licensing information added</TD>
 * </TR>
 * </TR>
 * <TR>
 * <TD>1.0.0.2</TD>
 * <TD>New SHA-1 Implementation based on OpenSSL SHA-1 source</TD>
 * </TR>
 * <TR>
 * <TD>1.0.0.0</TD>
 * <TD>Initial Version</TD>
 * </TR>
 * </TABLE>
 */

/**
 * @page Disclaimer
 *<center> <B>IMPORTANT NOTICE </B> </center>
 * Texas Instruments Incorporated and its subsidiaries (TI) reserve the right
 * to make corrections, modifications, enhancements, improvements, and other
 * changes to its products and services at any time and to discontinue any
 * product or service without notice. Customers should obtain the latest
 * relevant information before placing orders and should verify that such
 * information is current and complete. All products are sold subject to TIs
 * terms and conditions of sale supplied at the time of order acknowledgment.
 *
 * TI warrants performance of its hardware products to the specifications
 * applicable at the time of sale in accordance with TIs standard warranty.
 * Testing and other quality control techniques are used to the extent TI
 * deems necessary to support this warranty. Except where mandated by
 * government requirements, testing of all parameters of each product is not
 * necessarily performed.
 *
 * TI assumes no liability for applications assistance or customer product
 * design. Customers are responsible for their products and applications
 * using TI components. To minimize the risks associated with customer
 * products and applications, customers should provide adequate design and
 * operating safeguards.
 *
 * TI does not warrant or represent that any license, either express or
 * implied, is granted under any TI patent right, copyright, mask work right,
 * or other TI intellectual property right relating to any combination,
 * machine, or process in which TI products or services are used. Information
 * published by TI regarding third-party products or services does not
 * constitute a license from TI to use such products or services or a warranty
 * or endorsement thereof. Use of such information may require a license from a
 * third party under the patents or other intellectual property of the third
 * party, or a license from TI under the patents or other intellectual property
 * of TI.
 *
 * Reproduction of information in TI data books or data sheets is permissible
 * only if reproduction is without alteration and is accompanied by all
 * associated warranties, conditions, limitations, and notices. Reproduction
 * of this information with alteration is an unfair and deceptive business
 * practice. TI is not responsible or liable for such altered documentation.
 *
 * Resale of TI products or services with statements different from or beyond
 * the parameters stated by TI for that product or service voids all express
 * and any implied warranties for the associated TI product or service and is
 * an unfair and deceptive business practice. TI is not responsible or liable
 * for any such statements.
 *
 *
 * Mailing Address: <BR>
 * Texas Instruments <BR>
 * Post Office Box 655303 <BR>
 * Dallas, Texas 75265 <BR>
 *
 *
 * Copyright (C) 2008 Texas Instruments Incorporated - http://www.ti.com/ 
 * 
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * Information in this document is subject to change without notice.
 * Texas Instruments may have pending patent applications, trademarks,
 * copyrights, or other intellectual property rights covering matter in this
 * document.  The furnishing of this document is given for usage with Texas
 * Instruments products only and does not give you any license to the
 * intellectual property that might be contained within this document.
 * Texas Instruments makes no implied or expressed warranties in this document
 * and is not responsible for the products based from this document.
 *
 *
 */
